/*!
 * \file      uppertester_mbr_codec.hh
 * \brief     Header file for ITS UpperTester PKI codec definition.
 * \author    ETSI TTF T027
 * \copyright ETSI Copyright Notification
 *            No part may be reproduced except as authorized by written permission.
 *            The copyright and the foregoing restriction extend to reproduction in all media.
 *            All rights reserved.
 * \version   0.1
 */
#pragma once

#include <memory>

#include "codec.hh"
#include "params.hh"

class Base_Type;
class Record_Type;
class TTCN_Typedescriptor_t;
class TTCN_Buffer;

namespace LibItsMbr__TypesAndValues {
  class UtMbrTrigger;
  class UtMbrResults;
  class UtMbrEventInd;
} // namespace LibItsMbr__TypesAndValues

class uppertester_mbr_codec : public codec<Record_Type, Record_Type> {
public:
  static unsigned char c_utMbrInitialize;
  static unsigned char c_utMbrInitializeResult;
  static unsigned char c_utMbrTriggerResult;
  static unsigned char c_utMbrEventInd;
  static unsigned char c_utMbrTrigger;

private:
  int encode_(const Base_Type &type, const TTCN_Typedescriptor_t &field_descriptor, TTCN_Buffer &encoding_buffer);

public:
  uppertester_mbr_codec() : codec<Record_Type, Record_Type>(){};
  virtual ~uppertester_mbr_codec(){};

  virtual int                encode(const Record_Type &, OCTETSTRING &data);
  int                        encode(const LibItsMbr__TypesAndValues::UtMbrTrigger &, OCTETSTRING &data);
  int                        encode(const LibItsMbr__TypesAndValues::UtMbrEventInd &, OCTETSTRING &data);
  virtual int                decode(const OCTETSTRING &data, Record_Type &, params_its *params = NULL);
  std::unique_ptr<Base_Type> decode(const OCTETSTRING &data, params_its *params = NULL);
}; // End of class uppertester_mbr_codec
