#include "uppertester_mbr_layer_factory.hh"

#include "LibItsMbr_TestSystem.hh"
#include "LibItsMbr_TypesAndValues.hh"

#include "loggers.hh"

uppertester_mbr_layer::uppertester_mbr_layer(const std::string &p_type, const std::string &param)
  : t_layer<LibItsMbr__TestSystem::UpperTesterMbrPort>(p_type), _params(), _codec() {
  loggers::get_instance().log(">>> uppertester_mbr_layer::uppertester_mbr_layer: %s, %s", to_string().c_str(), param.c_str());
  // Setup parameters
  params::convert(_params, param);
  params::const_iterator it = _params.find(params::loopback);
  if (it == _params.cend()) {
    _params.insert(std::pair<std::string, std::string>(params::loopback, "0"));
  }
}

void uppertester_mbr_layer::sendMsg(const LibItsMbr__TypesAndValues::UtMbrInitialize &send_par, params &p_params) {
  loggers::get_instance().log_msg(">>> uppertester_mbr_layer::sendMsg", send_par);

  if (_params[params::loopback].compare("1") == 0) {
    LibItsMbr__TypesAndValues::UtMbrResults ut_mbr_results;
    ut_mbr_results.utMbrInitializeResult() = BOOLEAN(true);
    to_all_upper_ports(ut_mbr_results, p_params);
  } else {
    // Encode UpperTester PDU
    OCTETSTRING data;
    _codec.encode((const Record_Type &)send_par, data);
    // Update parameters
    send_data(data, p_params);
  }
}

void uppertester_mbr_layer::sendMsg(const LibItsMbr__TypesAndValues::UtMbrTrigger &send_par, params &p_params) {
  loggers::get_instance().log_msg(">>> uppertester_mbr_layer::sendMsg", send_par);

  if (_params[params::loopback].compare("1") == 0) {
    LibItsMbr__TypesAndValues::UtMbrResults ut_mbr_results;
    ut_mbr_results.utMbrTriggerResult() = BOOLEAN(true);
    to_all_upper_ports(ut_mbr_results, p_params);
  } else {
    // Encode UpperTester PDU
    OCTETSTRING data;
    _codec.encode(send_par, data);
    // Update parameters
    send_data(data, p_params);
  }
}

void uppertester_mbr_layer::sendMsg(const LibItsMbr__TypesAndValues::UtMbrEventInd &send_par, params &p_params) {
  loggers::get_instance().log_msg(">>> uppertester_mbr_layer::sendMsg", send_par);

  if (_params[params::loopback].compare("1") == 0) {
    LibItsMbr__TypesAndValues::UtMbrResults ut_mbr_results;
    ut_mbr_results.utMbrTriggerResult() = BOOLEAN(true);
    to_all_upper_ports(ut_mbr_results, p_params);
  } else {
    // Encode UpperTester PDU
    OCTETSTRING data;
    _codec.encode(send_par, data);
    // Update parameters
    send_data(data, p_params);
  }
}

void uppertester_mbr_layer::send_data(OCTETSTRING &data, params &p_params) {
  loggers::get_instance().log_msg(">>> uppertester_mbr_layer::send_data: ", data);
  // params.log();
  send_to_all_layers(data, p_params);
}

void uppertester_mbr_layer::receive_data(OCTETSTRING &data, params &p_params) {
  loggers::get_instance().log_msg(">>> uppertester_mbr_layer::receive_data: ", data);

  std::unique_ptr<Base_Type> r = _codec.decode(data);
  if (r.get() != nullptr) {
    // Pass it to the ports if any
    to_all_upper_ports(*r, p_params);
  }
}

uppertester_mbr_layer_factory uppertester_mbr_layer_factory::_f;
